<?php

namespace App\Base\Redis;

use Predis\Client;

class HostManualRepository
{
    protected $redis;

    protected $prefix = 'guard-manual|';

    public function __construct(Client $redis)
    {
        $this->redis = $redis;
    }

    public function getByKey(string $key): ?Host
    {
        $v = $this->redis->hgetall($key);
        if (empty($v)) return null;

        $kv = new Host();

        $identifier = explode($this->prefix, $key)[1];
        if (!$identifier) {
            return null;
        }

        $kv->setIdentifier($identifier);
        $kv->import($v);

        $ttl = $this->redis->ttl($key);
        if ($ttl > 0) {
            $dt = new \DateTime();
            $dt->setTimestamp(time() + $ttl);
            $kv->setExpiration(true);
            $kv->setExpirationAt($dt);
        } else {
            $kv->setExpiration(false);
        }
        return $kv;
    }

    public function getByIdentifier(string $identifier): ?Host
    {
        $key = $this->prefix.$identifier;
        return $this->getByKey($key);
    }

    public function delete(Host $kv): void
    {
        $this->redis->del([$this->prefix.$kv->getIdentifier()]);
        $this->redis->save();
    }

    public function deleteAll(): void
    {
        $arr = [];
        foreach ($this->redis->keys($this->prefix.'*') as $key) {
            $arr[] = $key;
        }

        if (!empty($arr)) {
            $this->redis->del($arr);
            $this->redis->save();
        }
    }

    public function save(Host $kv): void
    {
        $key = $this->prefix.$kv->getIdentifier();
        $this->redis->hmset($key, $kv->export());

        if ($kv->isExpiration()) {
            $expiration = $kv->getExpirationAt();
            if ($expiration) {
                $this->redis->expireat($key, $expiration->getTimestamp());
            }
        } else {
            $this->redis->persist($key);
        }

        $this->redis->save();
    }

    /**
     * @param string $query
     * @return Host[]
     */
    public function list(?string $query = null, int $page = 0, int $onPage = 10): array
    {
        $cursor = 0;
        $i = 0;

        /*
         * INVALID

        do {
            $result = $this->redis->scan(0, ['match' => $this->prefix . $query, 'count' => $onPage]);

            if ($page === $i) {
                return array_map(function($key) { return $this->getByKey($key); }, $result[1]);
            }
            $i++;
        } while ($cursor != 0);

        return [];
        */

        // TODO rewrite when this became problem
        $all_keys = $this->redis->keys($this->prefix.$query);
        $keys = array_slice($all_keys, $onPage*$page, $onPage);
        return array_map(function($key) { return $this->getByKey($key); }, $keys);
    }

    public function listAll(): array
    {
        return $this->list('*');
    }
}
